﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SampleApp.Common.Extensions;
using SampleApp.Data.Common;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.Business.Crud.Validators
{
    internal class PersonDuplicityValidator : ValidatorBase<Person>
    {
        public override void ValidateNew(Person entity, SampleAppContext context, DataValidationResult validationResult)
        {
            Validate(entity, context, validationResult, false);
        }

        public override void ValidateUpdate(Person entity, SampleAppContext context, DataValidationResult validationResult)
        {
            Validate(entity, context, validationResult, true);
        }

        private void Validate(Person entity, SampleAppContext context, DataValidationResult validationResult, bool isUpdate)
        {
            var duplicitEntities = context.People.Where(
                x => x.FirstName == entity.FirstName
                    && x.MiddleName == entity.MiddleName
                    && x.LastName == entity.LastName);

            if (isUpdate)
            {
                duplicitEntities = duplicitEntities.Where(x => x.Id != entity.Id);
            }

            var isDuplicit = duplicitEntities.Any();

            if (isDuplicit)
            {
                var message = "Osoba se jménem {0} {1} {2} již v databázi existuje."
                    .InvariantFormat(entity.FirstName, entity.MiddleName, entity.LastName);
                validationResult.AddError(message);
            }
        }
    }
}
