using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

using AutoMapper;

using SampleApp.Data.Common;
using SampleApp.Data.Context;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.Business.Queries
{
    internal abstract class QueryBase<T> : IQuery<T>
        where T : class, IPrimaryKey
    {
        private readonly ISampleAppContextFactory _contextFactory;

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
        protected QueryBase(ISampleAppContextFactory contextFactory)
        {
            _contextFactory = contextFactory;
        }

        protected SampleAppContext Context { get { return _contextFactory.ResolveContext(); } }

        public IEnumerable<T> GetAll(Func<IQueryable<T>, IQueryable<T>> queryExtensionFunc)
        {
            var query = GetQuery();
            query = queryExtensionFunc(query);

            // AsNoTracking - we don't need change tracking when we use it for one-way displaying
            // the result on the web, in stateless http request bound environment.
            // We can consider using tracked entities for more complex scenarios, but it this fragment isn't designed for it.
            return query.AsNoTracking().ToArray();
        }

        public T GetOne(int id)
        {
            var entity = GetQuery().SingleOrDefault(x => x.Id == id);
            return Mapper.Map<T>(entity);
        }

        protected abstract IQueryable<T> GetQuery();
    }
}