﻿using System;
using System.Collections.Generic;
using System.Linq;

using JetBrains.Annotations;

namespace SampleApp.Common.Extensions
{
    public class ObjectToStringFormatter
    {
        private readonly string _initialText = "";

        private readonly IDictionary<string, object> _values = new Dictionary<string, object>();

        public ObjectToStringFormatter()
        {
        }

        public ObjectToStringFormatter([NotNull] string initialText)
        {
            if (initialText == null)
                throw new ArgumentNullException("initialText");

            _initialText = initialText;
        }

        public ObjectToStringFormatter([NotNull] Type objectType)
        {
            if (objectType == null)
                throw new ArgumentNullException("objectType");

            _initialText = objectType.Name;
        }

        public ObjectToStringFormatter Property(string name, object value)
        {
            _values.Add(name, value);
            return this;
        }

        public override string ToString()
        {
            var text = string.Join(
                "; ",
                _values.Select(pair => "{0}: '{1}'".InvariantFormat(pair.Key, pair.Value ?? ">null<")));
            return string.IsNullOrEmpty(_initialText)
                ? text
                : string.Join(";", _initialText, text);
        }
    }
}
