﻿using System;
using System.Collections.Generic;

using JetBrains.Annotations;

using SampleApp.Common.Extensions;

namespace SampleApp.Data.Common
{
    public class DataValidationResult
    {
        private readonly List<IMessage> _messages = new List<IMessage>();

        public IEnumerable<IMessage> Messages
        {
            get { return _messages; }
        }

        public bool IsError { get; private set; }

        public void AddError(string text)
        {
            _messages.Add(new TextMessage(MessageType.Error, text));
            IsError = true;
        }

        public void AddError(string text, Exception exception)
        {
            _messages.Add(new TextMessage(MessageType.Error, text, exception));
            IsError = true;
        }

        public void AddError([NotNull] string messageTemplate, params object[] messageParameters)
        {
            if (messageTemplate == null)
                throw new ArgumentNullException("messageTemplate");

            AddError(messageTemplate.InvariantFormat(messageParameters));
        }

        public void AddMessage([NotNull] IMessage message)
        {
            if (message == null)
                throw new ArgumentNullException("message");

            if (message.Type == MessageType.Error)
                IsError = true;

            _messages.Add(message);
        }
    }
}
