﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using log4net;

using SampleApp.Common.Extensions;

namespace SampleApp.Data.Common
{
    public class TextMessage : IMessage
    {
        private static readonly ILog Logger = LogManager.GetLogger(typeof(TextMessage));

        public TextMessage(MessageType type, string text)
        {
            Type = type;
            Text = text;
        }

        public TextMessage(MessageType type, string text, Exception exception)
        {
            Type = type;
            Text = text;
            Exception = exception;
        }

        public string Text { get; set; }

        public MessageType Type { get; set; }

        public Exception Exception { get; set; }

        public virtual string ToHtml()
        {
            try
            {
                var html = "<span class='{0}'>{1}</span>".InvariantFormat(Type, Text);
                return html;
            }
            catch (Exception ex)
            {
                var msg = "Failed to format message: {0}.".InvariantFormat(Text);
                Logger.Error(msg, ex);
                string html = "<span class='{0}'>{1}</span>".InvariantFormat(MessageType.Error, msg);
                return html;
            }
        }

        public override string ToString()
        {
            return Text;
        }
    }
}
