using System;
using System.Collections.Generic;
using System.Data.Entity.Validation;
using System.Linq;
using System.Runtime.Serialization;

using SampleApp.Common.Exceptions;
using SampleApp.Common.Extensions;
using SampleApp.Data.Common;

namespace SampleApp.Data.Exceptions
{
    [Serializable]
    public class DataValidationException : SampleAppExceptionBase
    {
        private readonly DataValidationResult _result;

        public DataValidationException()
        {
        }

        public DataValidationException(string message)
            : base(message)
        {
            _result = new DataValidationResult();
            _result.AddError(message);
        }

        public DataValidationException(string message, Exception innerException)
            : base(message, innerException)
        {
            _result = new DataValidationResult();
            _result.AddError(message);
        }

        public DataValidationException(DataValidationResult result)
        {
            _result = result;
        }

        public DataValidationException(DbEntityValidationException innerException)
            : base("Chyba validace.", innerException)
        {
            _result = new DataValidationResult();
            WriteEfErrors(innerException.EntityValidationErrors);
        }

        public DataValidationException(IEnumerable<DbEntityValidationResult> entityValidationErrors)
        {
            _result = new DataValidationResult();
            WriteEfErrors(entityValidationErrors);
        }

        protected DataValidationException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public IEnumerable<IMessage> ValidationMessages
        {
            get { return _result.Messages; }
        }

        /// <summary>
        /// Gets a message that describes the current exception.
        /// </summary>
        /// <returns>
        /// The error message that explains the reason for the exception, or an empty string("").
        /// </returns>
        public override string Message
        {
            get
            {
                if (_result == null || !_result.Messages.Any())
                    return "General error";
                string text = null;
                foreach (var msg in _result.Messages)
                {
                    if (text != null)
                        text += "; ";
                    text += msg.ToString();
                }

                return text ?? "";
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2236:CallBaseClassMethodsOnISerializableTypes")]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            throw new NotSupportedException();
        }

        private void WriteEfErrors(IEnumerable<DbEntityValidationResult> entityValidationErrors)
        {
            foreach (var result in entityValidationErrors)
            {
                if (result.IsValid)
                {
                    continue;
                }

                foreach (var error in result.ValidationErrors)
                {
                    var message = "Chyba validace {0}.{1}: {2}".InvariantFormat(result.Entry.Entity, error.PropertyName, error.ErrorMessage);
                    _result.AddError(message);
                }
            }
        }
    }
}
