﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;

using log4net;
using log4net.Appender;
using log4net.Config;
using log4net.Layout;

using SampleApp.Common.Infrastructure;
using SampleApp.Data.Context;
using SampleApp.Data.SampleAppEntities;

namespace SampleApp.TestUtilities
{
    public abstract class DatabaseTestsBase : WindsorTestsBase
    {
        private ISampleAppContextFactory _contextFactory;
        private SampleAppContext _context;
        private TransactionScope _transaction;

        protected DatabaseTestsBase()
        {
            BasicConfigurator.Configure();
            var appender = LogManager.GetRepository()
                                     .GetAppenders()
                                     .OfType<ConsoleAppender>()
                                     .First();

            appender.Layout = new PatternLayout("%d %-5level %logger - %m%n"); // set pattern

            _transaction = new TransactionScope();
        }

        protected bool RunInTransaction
        {
            get
            {
                return _transaction != null;
            }

            set
            {
                if (!value && _transaction != null)
                {
                    _transaction.Dispose();
                    _transaction = null;
                }
                else if (value && _transaction == null)
                {
                    _transaction = new TransactionScope();
                }
            }
        }

        protected SampleAppContext Context
        {
            get
            {
                if (_contextFactory == null)
                    _contextFactory = WindsorStore.CurrentContainer.Resolve<ISampleAppContextFactory>();
                if (_context == null)
                    _context = _contextFactory.ResolveContext();

                return _context;
            }
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (_transaction != null)
                {
                    _transaction.Dispose();
                    _transaction = null;
                }
            }

            base.Dispose(disposing);
        }
    }
}
