﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using AutoMapper;

using Castle.Windsor;

using log4net;

using SampleApp.Common.Infrastructure;

namespace SampleApp.WebUI
{
    internal class AutoMapperConfig
    {
        private static readonly ILog Logger = LogManager.GetLogger(typeof(AutoMapperConfig));
        public static void Configure()
        {
            Mapper.Initialize(OnAutomapperInit);
            Mapper.AssertConfigurationIsValid();
            Mapper.Configuration.Seal();
        }

        private static void OnAutomapperInit(IConfiguration cfg)
        {
            var profiles = WindsorStore.CurrentContainer.ResolveAll<Profile>();

            // For Windsor injection to work with automapper components (e.g. resolvers):
            cfg.ConstructServicesUsing(type => WindsorStore.CurrentContainer.Resolve(type));

            foreach (var profile in profiles)
            {
                Logger.Info("Adding automapper profile: " + profile.ProfileName);
                cfg.AddProfile(profile);
                Logger.Info("Automapper profile configured: " + profile.ProfileName);
            }
        }
    }
}