﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using Castle.MicroKernel.Registration;
using Castle.MicroKernel.Resolvers.SpecializedResolvers;
using Castle.Windsor;
using Castle.Windsor.Configuration.Interpreters;

using SampleApp.Business.Infrastructure.Windsor;
using SampleApp.Common.Infrastructure;
using SampleApp.Data.Infrastructure.Windsor;
using SampleApp.WebUI.Infrastructure.Windsor;

namespace SampleApp.WebUI
{
    internal class WindsorConfig
    {
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000:Dispose objects before losing scope", Justification = "Disposed by WindsorStore.WindsorStore.TryStaticDispose()")]
        public static WindsorContainer InitializeContainer()
        {
            // XmlInterpreter reads AppDomain config file  (App.config or Web.config)
            var container = new WindsorContainer(new XmlInterpreter());
            RegisterTypes(container);
            RegisterControllers(container);

            // MVC Windsor plumbing:
            ControllerBuilder.Current.SetControllerFactory(new WindsorControllerFactory(container.Kernel));
            var resolver = new WindsorDependencyResolver(container);
            DependencyResolver.SetResolver(resolver);
            // Web API Windsor plumbing:
            ////GlobalConfiguration.Configuration.DependencyResolver = new WindsorDependencyResolver(container);
            ////GlobalConfiguration.Configuration.Services.Replace(typeof(IHttpControllerActivator), new WindsorCompositionRoot(container));

            WindsorStore.CurrentContainer = container;
            return container;
        }

        private static void RegisterControllers(WindsorContainer container)
        {
            // MVC controllers registration:
            container.Register(Classes.FromThisAssembly()
                            .BasedOn<IController>()
                            .LifestylePerWebRequest());

            // Web API controllers registration:
            ////container.Register(Classes.FromThisAssembly()
            ////                .BasedOn<IHttpController>()
            ////                .LifestylePerWebRequest());
        }

        private static void RegisterTypes(WindsorContainer container)
        {
            var kernel = container.Kernel;
            kernel.Resolver.AddSubResolver(new CollectionResolver(kernel));

            IWindsorInstaller[] installers =
                    {
                        new CommonWindsorInstaller(),
                        new DataWindsorInstaller(),
                        new BusinessWindsorInstaller(),
                        new WebUiWindsorInstaller()
                    };

            container.Install(installers);
        }
    }
}