﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using AutoMapper;

using SampleApp.Business.Queries;

namespace SampleApp.WebUI.Controllers
{
    public abstract class GridListControllerBase<TBusinessDto, TViewModel> : MvcControllerBase
        where TBusinessDto : class
        where TViewModel : class
    {
        protected GridListControllerBase(
            IQuery<TBusinessDto> query)
        {
            Query = query;
        }

        protected IQuery<TBusinessDto> Query { get; private set; }

        [HttpGet]
        public virtual ActionResult Index()
        {
            var dtoData = Query.GetAll(BuildListQuery);
            var result = Mapper.Map<IEnumerable<TViewModel>>(dtoData);
            return View(result);
        }

        /// <summary>
        /// Apply filtering, sorting and paging here.
        /// Be aware when no filtering - entire DB table would be loaded into application.
        /// </summary>
        /// <param name="dtoSource">the filtering, sorting, paging query conditions</param>
        /// <returns>final query to be sent to DB</returns>
        protected abstract IQueryable<TBusinessDto> BuildListQuery(IQueryable<TBusinessDto> dtoSource);
    }
}