﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Linq;

using AutoMapper;

using Castle.Facilities.TypedFactory;
using Castle.MicroKernel.Registration;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;

using SampleApp.Common.Authentication;
using SampleApp.Common.Interfaces;
using SampleApp.Data.Context;
using SampleApp.Data.SampleAppEntities;
using SampleApp.WebUI.Infrastructure.Authentication;

namespace SampleApp.WebUI.Infrastructure.Windsor
{
    public class WebUiWindsorInstaller : IWindsorInstaller
    {
        [SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1116:SplitParametersMustStartOnLineAfterDeclaration", Justification = "Reviewed. Suppression is OK here."), SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1118:ParameterMustNotSpanMultipleLines", Justification = "Reviewed. Suppression is OK here."), SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1118:ParameterMustNotSpanMultipleLines", Justification = "Reviewed. Suppression is OK here."), SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1115:ParameterMustFollowComma", Justification = "Reviewed. Suppression is OK here."), SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1115:ParameterMustFollowComma", Justification = "Reviewed. Suppression is OK here.")]
        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            if (container == null)
                throw new ArgumentNullException("container");

            if (store == null)
                throw new ArgumentNullException("store");

            container.Register(
                Classes.FromThisAssembly()
                .IncludeNonPublicTypes()
                .BasedOn<IService>()
                .WithServiceAllInterfaces()
                .WithServiceSelf()
                .AllowMultipleMatches(),

                Classes.FromThisAssembly()
                .IncludeNonPublicTypes()
                .BasedOn<Profile>()
                .WithServiceBase()
                .AllowMultipleMatches(),

            Component.For<SampleAppContext>()
            .ImplementedBy<SampleAppContext>()
            .OnCreate(OnCreateSampleAppContext)
                // .LifestyleScoped(),
            .LifestylePerWebRequest(),

            Component.For<SampleAppContext>()
            .ImplementedBy<SampleAppContext>()
            .OnCreate(OnCreateSampleAppContext)
            .Named("TransientContext")
            .LifestyleTransient(),

            Component.For<IAuthenticationService>().ImplementedBy<WebAuthenticationService>());

            if (!container.Kernel.GetFacilities().OfType<TypedFactoryFacility>().Any())
                container.AddFacility<TypedFactoryFacility>();
            container.Register(Component.For<ISampleAppContextFactory>().AsFactory());
        }

        public void OnCreateSampleAppContext(SampleAppContext context)
        {
            // TODO: configure what you need - timeouts, etc.
            context.Database.CommandTimeout = (int)TimeSpan.FromSeconds(20).TotalSeconds;
        }
    }
}