﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using SampleApp.Common.Infrastructure;
using SampleApp.Data.Aspects;
using SampleApp.Data.Context;
using SampleApp.Data.SampleAppEntities;

using Xunit;

namespace SampleApp.Demo.Tests.Demo
{
    public class DeadlockTests : DemoTestsBase
    {
        [Fact]
        public void NoDeadlockTest()
        {
            RunInTransaction = false;

            var operation1 = new UpdatMaiAndPersonlOperation();
            var operation2 = new UpdatePersonAndMailOperation();

            operation1.Execute();
            operation2.Execute();
        }

        [Fact]
        public void DeadlockTest()
        {
            RunInTransaction = false;

            var task1 = Task.Factory.StartNew(
                () =>
                {
                    for (var i = 0; i < 100; ++i)
                    {
                        try
                        {
                            var operation1 = new UpdatMaiAndPersonlOperation();
                            operation1.Execute();
                        }
                        catch (Exception ex)
                        {
                        }
                    }
                });
            var task2 = Task.Factory.StartNew(
                () =>
                {
                    for (var i = 0; i < 100; ++i)
                    {
                        try
                        {
                            var operation2 = new UpdatePersonAndMailOperation();
                            operation2.Execute();
                        }
                        catch (Exception ex)
                        {
                        }
                    }
                });

            Task.WaitAll(task1, task2);
        }

        private class UpdatePersonAndMailOperation
        {
            [DeadlockRetry]
            public void Execute()
            {
                var factory = WindsorStore.CurrentContainer.Resolve<ISampleAppContextFactory>();
                using (var context = factory.GetTransientContext())
                using (var transaction = context.Database.BeginTransaction())
                {
                    var person = context.People.Single(x => x.Id == 1);
                    person.Title = new Random().Next(1, 3).ToString();
                    context.SaveChanges();

                    var email = context.EmailAddresses.Single(x => x.Id == 1);
                    email.Mail = string.Format("test{0}@example.com", new Random().Next(1, 999));
                    context.SaveChanges();
                    transaction.Commit();
                }
            }
        }

        private class UpdatMaiAndPersonlOperation
        {
            [DeadlockRetry]
            public void Execute()
            {
                var factory = WindsorStore.CurrentContainer.Resolve<ISampleAppContextFactory>();
                using (var context = factory.GetTransientContext())
                using (var transaction = context.Database.BeginTransaction())
                {
                    var email = context.EmailAddresses.Single(x => x.Id == 1);
                    email.Mail = string.Format("test{0}@example.com", new Random().Next(1, 999));
                    context.SaveChanges();

                    var person = context.People.Single(x => x.Id == 1);
                    person.Title = new Random().Next(1, 3).ToString();
                    context.SaveChanges();
                    transaction.Commit();
                }
            }
        }
    }
}
