﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using AutoMapper;

using SampleApp.Data.SampleAppEntities;

using Xunit;

namespace SampleApp.Demo.Tests.Demo
{
    public class MappingTests : DemoTestsBase
    {
        [Fact]
        public void QueryTest()
        {
            ConfigureAutomapper();

            var browns1 = Context.People
                // .ToArray()
                .Select(p => new SmallPerson { Id = p.Id, FullName = p.FirstName + " " + p.LastName }).Where(sp => sp.FullName.Contains("Brown")).ToArray();

            var browns2 = Mapper.Map<IEnumerable<SmallPerson>>(Context.People).Where(sp => sp.FullName.Contains("Brown")).ToArray();

            var browns3 = Context.People.ProjectTo<SmallPerson>().Where(sp => sp.FullName.Contains("Brown")).ToArray();

            var browns4 = Context.People.UseAsDataSource().For<SmallPerson>().Where(sp => sp.FullName.Contains("Brown")).ToArray();
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            Mapper.Reset();
        }

        private void ConfigureAutomapper()
        {
            Mapper.Reset();
            Mapper.Initialize(cfg => cfg.AddProfile<TestProfile>());
            Mapper.AssertConfigurationIsValid();
            Mapper.Configuration.Seal();
        }

        private class TestProfile : Profile
        {
            protected override void Configure()
            {
                CreateMap<Person, SmallPerson>().ForMember(x => x.FullName, opts => opts.MapFrom(y => y.FirstName + " " + y.LastName));
                }
        }

        private class SmallPerson
        {
            public int Id { get; set; }

            public string FullName { get; set; }
        }
    }
}
