using System.Diagnostics.CodeAnalysis;
using System.Linq;

using Castle.Facilities.TypedFactory;
using Castle.MicroKernel.Registration;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Windsor;

using SampleApp.Common.Authentication;
using SampleApp.Data.Context;
using SampleApp.Data.SampleAppEntities;
using SampleApp.TestUtilities;

namespace SampleApp.Demo.Tests
{
    internal class DemoTestsInstaller : IWindsorInstaller
    {
        [SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1118:ParameterMustNotSpanMultipleLines", Justification = "Reviewed. Suppression is OK here."), SuppressMessage("StyleCop.CSharp.ReadabilityRules", "SA1115:ParameterMustFollowComma", Justification = "Reviewed. Suppression is OK here.")]
        public void Install(IWindsorContainer container, IConfigurationStore store)
        {
            container.Register(
            Component.For<IAuthenticationService>().ImplementedBy<MockAuthenticationService>(),

            Component.For<SampleAppContext>()
            .ImplementedBy<SampleAppContext>()
            .LifestylePerThread(),

            Component.For<SampleAppContext>()
            .ImplementedBy<SampleAppContext>()
            .Named("TransientContext")

            .LifestyleTransient());

            if (!container.Kernel.GetFacilities().OfType<TypedFactoryFacility>().Any())
                container.AddFacility<TypedFactoryFacility>();
            container.Register(Component.For<ISampleAppContextFactory>().AsFactory());
        }
    }
}